@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2011 Dave M
+# ClamTk, copyright (C) 2004-2012 Dave M
 #
 # This file is part of ClamTk (http://clamtk.sourceforge.net).
 #
@@ -12,7 +12,9 @@
 package ClamTk::GUI;
 
 use strict;
-# use warnings FATAL => 'all';
+# use warnings;
+# Can't use FATAL with warnings here;
+# chokes on the readline if the stop button is hit.
 $| = 1;
 
 use Cwd 'abs_path';
@@ -25,12 +27,19 @@ use Locale::gettext;
 use LWP::UserAgent;
 use File::Find::Rule;
 use Encode;
+use URI;
 
 use Gtk2;
 use Glib 'TRUE', 'FALSE';
 
-my $window;       # Top level window
-my $top_label;    # Status bar at the top
+# Somebody might complain about the "use bytes" thing,
+# but it's the only way I can get the RTL override malware
+# thingie to JUST WORK(TM).
+use bytes;
+binmode( STDIN,  ':encoding(UTF-8)' );
+binmode( STDOUT, ':encoding(UTF-8)' );
+
+my $window;    # Top level window
 
 my $directive = '';    # Arguments passed to clamscan
 my %dirs_scanned;      # Keeps track of directories scanned for display
@@ -44,18 +53,25 @@ my $found;             # Stores info about possibly infected files
 my $gui_tb;            # Easy buttons bar
 my $start_time;        # For use with Elapsed time
 my $scan_frame;        # Holds the progressbar, files scanned, etc.
+my $drop_frame;        # For the drag and drop stuff
 my $found_count = 0;   # Number of threats found
 my $num_scanned = 0;   # Number of files scanned, clean or bad
 my $av_remote   = '';  # ClamAV version, according to remote servers
 my $proxy_status_img;       # Image; global for 'flicker' appearance
-my ($last_infect_date);     # Date of last known threat found
 my ($scan_status_label);    # Shows what's being scanned, a little more
 
+# A little color never hurt anyone.  Standards be damned.
+# my $silver = Gtk2::Gdk::Color->new( 0xCCCC, 0xCCCC, 0xCCCC );
+my $white = Gtk2::Gdk::Color->new( 0xFFFF, 0xFFFF, 0xFFFF );
+
+# Last scan label (should be an InfoBar, but can't find it)
+my $last_scan_bar = Gtk2::Label->new;
+
 # Scan frame stuff, under the progressbar
 my ( $left_status, $mid_status, $right_status );
 
 # Right-most column
-my ( $engine_version_text, $defs_version_text, $last_date_text );
+my ( $engine_version_text, $defs_version_text );
 
 # Left-most column - global so we can adjust them throughout
 my ( $engine_gui_img, $defs_gui_img, $status_gui_img );
@@ -69,7 +85,7 @@ sub start_gui {
     $window = Gtk2::Window->new();
     $window->signal_connect( destroy => sub { Gtk2->main_quit; } );
     $window->set_title( gettext('Virus Scanner') );
-    $window->set_border_width(5);
+    $window->set_border_width(0);
     $window->set_resizable(FALSE);
     $window->set_position('center');
     if ( -e '/usr/share/pixmaps/clamtk.png' ) {
@@ -143,6 +159,12 @@ sub start_gui {
             undef, 	sub { history('delete') },
             FALSE
         ],
+        # Show last scan info (date of scan and last infected)
+        [   'LastScanInfo', 		undef,
+            gettext('_Last Scan Information'), "<control>L",
+            undef, 	\&last_scan,
+            FALSE
+        ],
         # Closes the scan bar, zeroes out everything
         [   'ClearOutput',                undef,
             gettext('Clear _Output'),     '<control>O',
@@ -184,7 +206,7 @@ sub start_gui {
 	    FALSE
         ],
 
-        ### The final three here are for Advanced ###
+        ### The final four here are for Advanced ###
         # Schedule scans and AV sig updates
         [   'Scheduler', 		undef,
             gettext('_Scheduler'), 	'<control>T', undef,	
@@ -228,6 +250,7 @@ sub start_gui {
          </menu>
           <menu action='ViewMenu'>
           <menuitem action='ManageHistories'/>
+          <menuitem action='LastScanInfo'/>
           <menuitem action='ClearOutput'/>
          </menu>
          <menu action='QuarantineMenu'>
@@ -261,13 +284,18 @@ sub start_gui {
     # We have to declare this now for items in the Actions frame
     my $tt = Gtk2::Tooltips->new();
 
-    $top_label = Gtk2::Label->new;
-    $vbox->pack_start( $top_label, FALSE, FALSE, 5 );
+    # Last scan bar area
+    # Shows things like "Please wait..."
+    # We have to put it at the top now -
+    # sometimes does not show when at the bottom
+    # and beginning a scan
+    $vbox->pack_start( $last_scan_bar, FALSE, FALSE, 2 );
 
     # Actions Frame
     # These are for easy access to buttons for scanning
 
-    my $gui_frame = Gtk2::Frame->new( gettext('Actions') );
+    #my $gui_frame = Gtk2::Frame->new( gettext('Actions') );
+    my $gui_frame = Gtk2::Frame->new();
     $vbox->pack_start( $gui_frame, FALSE, FALSE, 0 );
     $gui_frame->set_shadow_type('none');
 
@@ -281,7 +309,6 @@ sub start_gui {
     # The toolbar ($gui_tb) is global so we can change its style
     # from the Preferences dialog.
     $gui_tb = Gtk2::Toolbar->new;
-    $gui_tb->set_border_width(5);
     $gui_tb->set_style('both');
     $gui_tb->set_show_arrow(FALSE);
 
@@ -296,25 +323,23 @@ sub start_gui {
 
     $gui_tb->insert( Gtk2::SeparatorToolItem->new, -1 );
 
-    # Button for scanning just a file
-    my $file_img = Gtk2::ToolButton->new_from_stock('gtk-file');
-    $file_img->set_expand(TRUE);
-    $file_img->set_label( gettext('File') );
-    $file_img->signal_connect( clicked => sub { ClamTk::GUI->getfile('file') }
-    );
-    $tt->set_tip( $file_img, gettext('Scan a file') );
-    $gui_tb->insert( $file_img, -1 );
+    # Button for histories
+    my $hist_img = Gtk2::ToolButton->new_from_stock('gtk-edit');
+    $hist_img->set_expand(TRUE);
+    $hist_img->set_label( gettext('Histories') );
+    $hist_img->signal_connect( clicked => sub { history('delete') } );
+    $tt->set_tip( $hist_img, gettext('View your previous scans') );
+    $gui_tb->insert( $hist_img, -1 );
 
     $gui_tb->insert( Gtk2::SeparatorToolItem->new, -1 );
 
-    # Button for scanning a directory
-    my $dir_img = Gtk2::ToolButton->new_from_stock('gtk-directory');
-    $dir_img->set_expand(TRUE);
-    $dir_img->set_label( gettext('Directory') );
-    $dir_img->signal_connect( clicked => sub { ClamTk::GUI->getfile('dir') }
-    );
-    $tt->set_tip( $dir_img, gettext('Scan a directory') );
-    $gui_tb->insert( $dir_img, -1 );
+    # Button for preferences
+    my $pref_img = Gtk2::ToolButton->new_from_stock('gtk-preferences');
+    $pref_img->set_expand(TRUE);
+    $pref_img->set_label( gettext('Preferences') );
+    $pref_img->signal_connect( clicked => \&preferences );
+    $tt->set_tip( $pref_img, gettext('Set or view your preferences') );
+    $gui_tb->insert( $pref_img, -1 );
 
     $gui_tb->insert( Gtk2::SeparatorToolItem->new, -1 );
 
@@ -332,11 +357,12 @@ sub start_gui {
     # This shows versions of ClamAV, ClamTk, signatures,
     # date of last scan, date of last infected file
 
-    my $status_frame = Gtk2::Frame->new( gettext('Status') );
-    $vbox->pack_start( $status_frame, FALSE, FALSE, 0 );
+    #my $status_frame = Gtk2::Frame->new( gettext('Status') );
+    my $status_frame = Gtk2::Frame->new('		');
+    $vbox->pack_start( $status_frame, FALSE, FALSE, 5 );
     $status_frame->set_shadow_type('none');
 
-    my $status_table = Gtk2::Table->new( 5, 3, FALSE );
+    my $status_table = Gtk2::Table->new( 3, 3, FALSE );
     $status_frame->add($status_table);
 
     # Version of ClamAV
@@ -374,51 +400,29 @@ sub start_gui {
     $defs_gui_box->pack_start( $defs_version_text, TRUE, TRUE, 0 );
     $status_table->attach_defaults( $defs_gui_box, 0, 3, 2, 3 );
 
-    # Last virus scan date
-    my $last_gui_box = Gtk2::HBox->new( TRUE, 0 );
-    my $last_text = Gtk2::Label->new( gettext('Last scan') );
-    $last_date_text = Gtk2::Label->new( gettext('Unknown') );
-    my $last_gui_img =
-        Gtk2::Image->new_from_stock( 'gtk-find', 'small-toolbar' );
-    $tt->set_tip( $last_gui_img, gettext('Date of your last scan') );
-    $last_gui_box->pack_start( $last_gui_img,   TRUE, TRUE, 0 );
-    $last_gui_box->pack_start( $last_text,      TRUE, TRUE, 0 );
-    $last_gui_box->pack_start( $last_date_text, TRUE, TRUE, 0 );
-    $status_table->attach_defaults( $last_gui_box, 0, 3, 3, 4 );
-
-    # Last infected file date
-    my $last_infect_box = Gtk2::HBox->new( TRUE, 0 );
-    my $last_infect_img =
-        Gtk2::Image->new_from_stock( 'gtk-find', 'small-toolbar' );
-    my $last_infect_text = Gtk2::Label->new( gettext('Last infected file') );
-    $last_infect_date = Gtk2::Label->new( gettext('Unknown') );
-    $tt->set_tip( $last_infect_img, gettext('Date of last known infection') );
-    $last_infect_box->pack_start( $last_infect_img,  TRUE, TRUE, 0 );
-    $last_infect_box->pack_start( $last_infect_text, TRUE, TRUE, 0 );
-    $last_infect_box->pack_start( $last_infect_date, TRUE, TRUE, 0 );
-    $status_table->attach_defaults( $last_infect_box, 0, 3, 4, 5 );
-
     # Scan frame:
     # This holds the progressbar and keeps track of files
     # scanned, viruses found and the status/elapsed time
 
-    $scan_frame = Gtk2::Frame->new( gettext('Scan') );
-    $vbox->pack_start( $scan_frame, FALSE, FALSE, 0 );
+    #$scan_frame = Gtk2::Frame->new( gettext('Scan') );
+    $scan_frame = Gtk2::Frame->new();
+    $vbox->pack_start( $scan_frame, FALSE, FALSE, 10 );
 
     my $scan_box = Gtk2::VBox->new();
+    $scan_frame->modify_bg( 'normal', $white );
     $scan_frame->add($scan_box);
 
     $scan_status_label = Gtk2::Label->new('');
     $scan_status_label->set_justify('center');
     $scan_status_label->set_ellipsize('middle');
-    $scan_box->pack_start( $scan_status_label, FALSE, FALSE, 0 );
+    $scan_box->pack_start( $scan_status_label, FALSE, FALSE, 2 );
 
     my $scan_table = Gtk2::Table->new( 1, 2, FALSE );
     $scan_box->pack_start( $scan_table, FALSE, FALSE, 0 );
 
     $pb = Gtk2::ProgressBar->new;
-    $scan_table->attach( $pb, 0, 1, 0, 1, [qw/fill expand/], [qw/shrink/], 0,
-        0 );
+    $scan_table->attach( $pb, 0, 1, 0, 1, [ 'fill', 'expand' ],
+        ['shrink'], 0, 0 );
     $pb->set_fraction(0);
 
     my $stop_image =
@@ -427,13 +431,24 @@ sub start_gui {
     $stop_btn->set_property( image => $stop_image );
     $stop_btn->set_relief('none');
     $tt->set_tip( $stop_btn, gettext('Stop scanning now') );
-    $scan_table->attach( $stop_btn, 1, 2, 0, 1, [qw/shrink shrink/],
-        [qw/shrink/], 0, 0 );
+    $scan_table->attach( $stop_btn, 1, 2, 0, 1, [ 'shrink', 'shrink' ],
+        ['shrink'], 0, 0 );
     $stop_btn->signal_connect(
         'clicked' => sub {
+            clear_output(), return if ($stopped);
+
             kill 15, $scan_pid if ($scan_pid);
-            $scan_status_label->set_text( gettext('Please wait...') );
             waitpid( $scan_pid, 0 ) if ($scan_pid);
+            please_wait();
+
+            my $loop = Glib::MainLoop->new;
+            Glib::Timeout->add(
+                2000,
+                sub {
+                    $loop->quit;
+                    FALSE;
+                } );
+            $loop->run;
 
             # this close returns the stupid readline() error.
             # not sure how to fix it yet, besides commenting
@@ -441,6 +456,8 @@ sub start_gui {
             # stop the $SCAN so far...
             close($SCAN);    # or warn "Unable to close scanner! $!\n";
             $scan_status_label->set_text('');
+            $last_scan_bar->set_text('');
+            $last_scan_bar->hide();
             $stopped = 1;
         } );
 
@@ -456,9 +473,57 @@ sub start_gui {
     $right_status = Gtk2::Label->new( gettext('Ready') );
     $bottom_box->pack_start( $right_status, TRUE, TRUE, 5 );
 
+    # Drag-n-drop frame
+    $drop_frame = Gtk2::Frame->new();
+    $vbox->pack_start( $drop_frame, TRUE, TRUE, 2 );
+    my $dropbox = Gtk2::EventBox->new;
+    $dropbox->modify_bg( 'normal', $white );
+    $drop_frame->add($dropbox);
+    my $droplabel = Gtk2::Label->new;
+    $droplabel->set_markup( "<span foreground = '#FFFFFF' size='medium'>"
+            . gettext('Drag and drop')
+            . "</span>" );
+    $dropbox->add($droplabel);
+    $dropbox->signal_connect(
+        'enter-notify-event' => sub {
+            $droplabel->set_markup(
+                      "<span foreground = '#CCCCCC' size='medium'>"
+                    . gettext('Drag and drop')
+                    . "</span>" );
+        } );
+    $dropbox->signal_connect(
+        'leave-notify-event' => sub {
+            $droplabel->set_markup(
+                      "<span foreground = '#FFFFFF' size='medium'>"
+                    . gettext('Drag and drop')
+                    . "</span>" );
+        } );
+
+    $dropbox->drag_dest_set( [ 'drop', 'motion', 'highlight' ],
+        [ 'copy', 'private', 'default', 'move', 'link', 'ask' ] );
+    $dropbox->signal_connect(
+        drag_data_received => sub {
+            my ( $widget, $context, $widget_x, $widget_y, $data, $info,
+                $time )
+                = @_;
+            my $u     = URI->new( $data->data );
+            my $upath = $u->file;
+            #$upath =~ s|%([0-9a-zA-Z]{2})|chr(hex($1))|ge;
+            #$upath =~ s|file://(.*?)|$1|;
+            if ( -e $upath ) {
+                ClamTk::GUI->getfile( 'cmd-scan', $upath );
+            }
+        } );
+
+    my $target_list = Gtk2::TargetList->new();
+    my $atom        = Gtk2::Gdk::Atom->new("text/uri-list");
+    $target_list->add( $atom, 0, 0 );
+    $dropbox->drag_dest_set_target_list($target_list);
+
     $window->set_focus_child($stop_btn);
     $window->show_all();
     $scan_frame->hide;
+    $last_scan_bar->hide;
 
     # The only reason we take @ARGV is for right-click scanning,
     # not arguments (e.g., -v).
@@ -557,7 +622,10 @@ sub start_gui {
 sub update_status_frame {
     # This is useful as its own subroutine so we
     # can call it from anywhere
-    $top_label->set_text( gettext('Please wait, initializing...') );
+    $last_scan_bar->set_markup( "<span background = '#FFFFC8'>"
+            . gettext('Please wait, initializing...')
+            . "</span>" );
+    $last_scan_bar->show_all();
     Gtk2->main_iteration while ( Gtk2->events_pending );
 
     # ClamAV version
@@ -586,22 +654,8 @@ sub update_status_frame {
     # Signatures
     set_sig_status();
 
-    # Last scan
-    my $last_scan = ClamTk::App->lastscan();
-    if ($last_scan) {
-        $last_date_text->set_text($last_scan);
-    } else {
-        $last_date_text->set_text( gettext('Never') );
-    }
-
-    # Last infection
-    my $last_infection = ClamTk::Prefs->get_preference('LastInfection');
-    if ($last_infection) {
-        $last_infect_date->set_text($last_infection);
-    } else {
-        $last_infect_date->set_text( gettext('Never') );
-    }
-    $top_label->set_text('');
+    $last_scan_bar->set_text('');
+    $last_scan_bar->hide();
     return;
 }
 
@@ -839,10 +893,14 @@ sub getfile {
         push( @scans, $paths->{directory} );
         if ( !$prefs{Recursive} ) {
             $directive .= ' --max-dir-recursion=1';
+        } elsif ( $prefs{Recursive} ) {
+            $directive .= ' --recursive';
         }
-        $top_label->set_text( gettext('Please wait...') );
-        @files = $rule->in( $paths->{directory} );
+        $window->queue_draw;
+        please_wait();
+        $window->queue_draw;
         Gtk2->main_iteration while ( Gtk2->events_pending );
+        @files = $rule->in( $paths->{directory} );
     } elsif ( $option eq 'file' ) {
         $dialog = Gtk2::FileChooserDialog->new(
             gettext($title), $window, 'open',
@@ -852,7 +910,7 @@ sub getfile {
         $dialog->set_select_multiple(TRUE);
         $dialog->set_position('center-on-parent');
         if ( "ok" eq $dialog->run ) {
-            $top_label->set_text( gettext('Please wait...') );
+            please_wait();
             $window->queue_draw;
             Gtk2->main_iteration while ( Gtk2->events_pending );
             push( @scans, $dialog->get_filenames );
@@ -870,7 +928,7 @@ sub getfile {
         @files = $rule->in($dir);
         push( @scans, $dir );
     } elsif ( $option eq 'full-home' ) {
-        $top_label->set_text( gettext('Please wait...') );
+        please_wait();
         Gtk2->main_iteration while ( Gtk2->events_pending );
         push( @scans, $paths->{directory} );
         $directive .= ' --recursive';
@@ -899,7 +957,7 @@ sub getfile {
                 clean_up('permissions');
                 return;
             }
-            $top_label->set_text( gettext('Please wait...') );
+            please_wait();
             Gtk2->main_iteration while ( Gtk2->events_pending );
             $window->queue_draw;
             $dialog->destroy;
@@ -912,18 +970,16 @@ sub getfile {
             if ( !$prefs{Recursive} ) {
                 $directive .= ' --max-dir-recursion=1'
                     if ( $option ne 'recur' );
-            }
-            if ( $option eq 'recur' ) {
+            } elsif ( $prefs{Recursive} || $option eq 'recur' ) {
                 $directive .= ' --recursive';
             }
-            Gtk2->main_iteration while ( Gtk2->events_pending );
         } else {
             $dialog->destroy;
             return;
         }
     } elsif ( $option eq 'cmd-scan' ) {
         if ( -d $cmd_input[0] ) {
-            $top_label->set_text( gettext('Please wait...') );
+            please_wait();
             $window->queue_draw;
             Gtk2->main_iteration while ( Gtk2->events_pending );
             @files = $rule->in( $cmd_input[0] );
@@ -931,23 +987,25 @@ sub getfile {
             if ( !$prefs{Recursive} ) {
                 $directive .= ' --max-dir-recursion=1'
                     if ( $option ne 'recur' );
+            } elsif ( $prefs{Recursive} ) {
+                $directive .= ' --recursive';
             }
         } else {
-            $top_label->set_text( gettext('Please wait...') );
+            please_wait();
             $window->queue_draw;
             Gtk2->main_iteration while ( Gtk2->events_pending );
-            push( @scans, @cmd_input );
-            if ( !$prefs{Recursive} ) {
-                $directive .= ' --max-dir-recursion=1'
-                    if ( $option ne 'recur' );
+            for (@cmd_input) {
+                push( @scans, $_ ) if ( -f $_ );
             }
-            @files = @cmd_input;
+            $directive .= ' --max-dir-recursion=1';
+            @files = @scans;
         }
     } else {
         warn "Unknown option '$option'.\n";
-        return;
     }
 
+    # close the drag and drop frame
+    $drop_frame->hide();
     # open the scan frame for display
     $scan_frame->show_all();
     Gtk2->main_iteration while ( Gtk2->events_pending );
@@ -1032,6 +1090,10 @@ sub filter {
             }
         }
     }
+
+    # Remove empty files - ClamAV ignores them.
+    # Really, this only affects the counting.
+    @$file_ref = grep { -s $_ } @$file_ref;
 }
 
 sub scan {
@@ -1039,9 +1101,12 @@ sub scan {
     # @get		 = list of files/paths sent to clamscan
     my ( $directive, @get ) = @_;
     @get = map { quotemeta($_) } @get;
-
     $window->queue_draw;
 
+    # Leave if we have no direct files or directory to scan
+    return if ( !@get );
+
+    # Update the timer
     timer();
 
     my $paths   = ClamTk::App->get_path('all');
@@ -1079,7 +1144,8 @@ sub scan {
         }
         next if (/^LibClamAV/);
         next if (/^\s*$/);
-        $top_label->set_text('');
+        $last_scan_bar->set_text('');
+        $last_scan_bar->hide();
 
         my ( $file, $status );
         if (/(.*?): ([^:]+) FOUND/) {
@@ -1088,9 +1154,8 @@ sub scan {
         } elsif (/(.*?): (OK)$/) {
             $file   = $1;
             $status = $2;
-        }    # else {
-             #	warn "something else: file = <$file>, stat = <$status>\n";
-             # }
+        }    #     else {
+             #    	warn "something else: file = <$file>, stat = <$status>\n";
              #}
 
         # Ensure the file is still there (things get moved)
@@ -1159,6 +1224,7 @@ sub scan {
         } else {
             $current += $step;
         }
+        Gtk2->main_iteration while ( Gtk2->events_pending );
         $pb->set_fraction($current);
         $pb->set_text( sprintf gettext('Percent complete: %2d'),
             $current * 100 );
@@ -1177,7 +1243,7 @@ sub scan {
 }
 
 sub clear_output {
-    # Return if there are files being scanned
+    # Return if there are files still being scanned
     return if ( !$stopped );
 
     # Reset the progressbar
@@ -1185,7 +1251,8 @@ sub clear_output {
     $pb->set_text('');
 
     # Clear the text
-    $top_label->set_text('');
+    $last_scan_bar->set_text('');
+    $last_scan_bar->hide();
     $scan_status_label->set_text('');
 
     # Refresh the main window
@@ -1198,6 +1265,9 @@ sub clear_output {
 
     # Hide the scanning frame
     $scan_frame->hide();
+
+    # Show the drag and drop frame
+    $drop_frame->show_all();
     Gtk2->main_iteration while ( Gtk2->events_pending );
     $window->queue_draw;
     return;
@@ -1223,7 +1293,8 @@ sub clean_up {
 
     # ...and remove the text
     $pb->set_text("");
-    $top_label->set_text('');
+    $last_scan_bar->set_text('');
+    $last_scan_bar->hide();
 
     my $db_total = ClamTk::App->get_num_sigs();
     my $REPORT;    # filehandle for histories log
@@ -1231,7 +1302,6 @@ sub clean_up {
 
     # Save date of scan
     if ( $found_count > 0 ) {
-        $last_infect_date->set_text("$day $mon $year");
         ClamTk::Prefs->set_preference( 'LastInfection', "$day $mon $year" );
         $window->queue_draw;
     }
@@ -1307,10 +1377,6 @@ sub clean_up {
     print $REPORT '-' x ( $lsize + $rsize + 5 ), "\n";
     close($REPORT);
 
-    # Update the Last scan field
-    my $lastdate = ClamTk::App->lastscan();
-    $last_date_text->set_text($lastdate);
-
     # If threats are found, show the Results window.
     if ($found_count) {
         ClamTk::Results->display($found);
@@ -1371,6 +1437,12 @@ sub maintenance {
     $restore_btn->signal_connect(
         clicked => sub { restore( \@q_files ), 'false_pos' } );
 
+    my $del_pos_btn = Gtk2::ToolButton->new_from_stock('gtk-delete');
+    $del_pos_btn->set_is_important(TRUE);
+    $new_bar->insert( $del_pos_btn, -1 );
+    $del_pos_btn->signal_connect(
+        clicked => sub { main_del_pos( \@q_files ), 'delete' } );
+
     my $m_sep = Gtk2::SeparatorToolItem->new;
     $m_sep->set_draw(FALSE);
     $m_sep->set_expand(TRUE);
@@ -1381,12 +1453,6 @@ sub maintenance {
     $new_bar->insert( $pos_quit_btn, -1 );
     $pos_quit_btn->signal_connect( clicked => sub { $main_win->destroy } );
 
-    my $del_pos_btn = Gtk2::ToolButton->new_from_stock('gtk-delete');
-    $del_pos_btn->set_is_important(TRUE);
-    $new_bar->insert( $del_pos_btn, -1 );
-    $del_pos_btn->signal_connect(
-        clicked => sub { main_del_pos( \@q_files ), 'false_pos' } );
-
     for my $opt (@q_files) {
         push @{ $new_slist->{data} }, basename($opt);
     }
@@ -1408,18 +1474,8 @@ sub restore {
     my $full_path = $top_dir .= $tdata;
     return if ( not exists $data->[0] );
 
-    # Slurp in the file for md5-ing:
-    my $ctx = do {
-        local $/ = undef;
-        open( my $F, '<', $full_path ) or do {
-            show_message_dialog( $window, 'warning', 'ok',
-                'Cannot open ' . $full_path . ": $!\n" );
-            return;
-        };
-        binmode($F);
-        <$F>;
-    };
-    my $md5 = md5_hex($ctx);
+    my $md5 = get_md5($full_path);
+    return if ( !$md5 );
 
     # By default, the final destination will be $HOME
     my $final_destination = $paths->{directory} . '/';
@@ -1493,18 +1549,7 @@ sub main_del_pos {
     my $full_path = $top_dir .= $data->[0];
     return if ( !-e $full_path );
 
-    # Slurp in the file for md5-ing:
-    my $ctx = do {
-        local $/ = undef;
-        open( my $F, '<', $full_path ) or do {
-            show_message_dialog( $window, 'warning', 'ok',
-                'Cannot open ' . $full_path . " for md5-ing: $!\n" );
-            return;
-        };
-        binmode($F);
-        <$F>;
-    };
-    my $md5 = md5_hex($ctx);
+    my $md5 = get_md5($full_path);
 
     # Update restore file: remove the file from it
     ClamTk::Prefs->restore( $md5, 'remove' );
@@ -1587,19 +1632,7 @@ sub del_quarantined {
                 foreach my $f (@trash) {
                     my $full_path = $paths . "/";
                     $full_path .= $f;
-                    my $ctx = do {
-                        local $/ = undef;
-                        open( my $F, '<', $full_path ) or do {
-                            show_message_dialog( $window, 'warning', 'ok',
-                                      'cannot open '
-                                    . $full_path
-                                    . " for md5-ing: $!\n" );
-                            next;
-                        };
-                        binmode($F);
-                        <$F>;
-                    };
-                    my $md5 = md5_hex($ctx);
+                    my $md5 = get_md5($full_path);
 
                     # update restore file: remove the file from it
                     # if it exists there
@@ -1734,10 +1767,6 @@ sub history_del_single {
     splice @{ $new_hlist->{data} }, $deref, 1;
     @$h_ref = glob "$history/*";
 
-    # Update the Last scan field
-    my $lastdate = ClamTk::App->lastscan();
-    $last_date_text->set_text($lastdate);
-
     return;
 }
 
@@ -2061,13 +2090,13 @@ sub preferences {
 
     $man_table->attach_defaults( $ip_label,   0, 1, 0, 1 );
     $man_table->attach_defaults( $ip_example, 1, 2, 0, 1 );
-    $man_table->attach( $ip_box, 2, 3, 0, 1, [qw/expand expand/],
-        [qw/shrink/], 0, 0 );
+    $man_table->attach( $ip_box, 2, 3, 0, 1, [ 'expand', 'expand' ],
+        ['shrink'], 0, 0 );
 
     $man_table->attach_defaults( $port_label,   0, 1, 1, 2 );
     $man_table->attach_defaults( $port_example, 1, 2, 1, 2 );
-    $man_table->attach( $port_box, 2, 3, 1, 2, [qw/shrink shrink/],
-        [qw/shrink/], 0, 0 );
+    $man_table->attach( $port_box, 2, 3, 1, 2, [ 'shrink', 'shrink' ],
+        ['shrink'], 0, 0 );
 
     $proxy_btn->signal_connect(
         toggled => sub {
@@ -2306,9 +2335,70 @@ sub window_coords {
     return $window->get_position;
 }
 
+sub last_scan {
+    my $last_scan      = ClamTk::App->lastscan();
+    my $last_infection = ClamTk::Prefs->get_preference('LastInfection');
+    $last_scan_bar->show_all();
+
+    # We're trying to mimic the InfoBar color here...
+    # Doesn't seem quite right yet, but it works for now.
+    # Would be much easier to use it, but apparently it's
+    # not packaged in perl-Gtk2 anymore?
+
+    for my $q (qw ( 1 2)) {
+        $last_scan_bar->set_markup(
+            ( $q == 1 )
+            ? "<span background = '#FFFFC8'>"
+                . ( sprintf gettext('Date of your last scan: %s'),
+                $last_scan )
+                . "</span>"
+            : "<span background = '#FFFFC8'>"
+                . (
+                sprintf gettext('Date of last known infection: %s'),
+                $last_infection
+                )
+                . "</span>"
+                );
+
+        my $loop = Glib::MainLoop->new;
+        Glib::Timeout->add(
+            3000,
+            sub {
+                $loop->quit;
+                FALSE;
+            } );
+        $loop->run;
+    }
+    $last_scan_bar->hide();
+    return;
+}
+
+sub please_wait {
+    $last_scan_bar->set_markup( "<span background = '#FFFFC8'>"
+            . gettext('Please wait...')
+            . "</span>" );
+    $last_scan_bar->show_all();
+    return;
+}
+
+sub get_md5 {
+    my $slurp = shift;
+    my $ctx   = do {
+        local $/ = undef;
+        open( my $F, '<', $slurp ) or do {
+            show_message_dialog( $window, 'warning', 'ok',
+                'Cannot open ' . $slurp . ": $!\n" );
+            return;
+        };
+        binmode($F);
+        <$F>;
+    };
+    return md5_hex($ctx);
+}
+
 sub about {
     my $about = Gtk2::AboutDialog->new;
-    $about->set_authors("Dave M, 2004-2011\ndave.nerd <at> gmail.com");
+    $about->set_authors("Dave M, 2004-2012\ndave.nerd <at> gmail.com");
     $about->set_version( ClamTk::App->get_TK_version() );
     my $contributors = 'Please see http://clamtk.sf.net/credits.html';
     $about->set_translator_credits($contributors);
