@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2011 Dave M
+# ClamTk, copyright (C) 2004-2012 Dave M
 #
 # This file is part of ClamTk (http://clamtk.sourceforge.net).
 #
@@ -26,18 +26,24 @@ use Glib 'TRUE', 'FALSE';
 use Locale::gettext;
 use POSIX 'locale_h';
 
+use bytes;
+binmode( STDIN,  ':utf8' );
+binmode( STDOUT, ':utf8' );
+
 my $slist;
 my $hover_label;
 my $hash;
 
+# We have to avoid parsing mail for now:
+# http://clamtk.sourceforge.net/faq.html#inbox
+my $maildirs = join( '|',
+    '.thunderbird', '.mozilla-thunderbird', '.evolution',
+    'Mail',         'kmail',                "\.pst" );
+
 sub display {
     shift;    # don't need package name
     ($hash) = @_;
 
-    for my $k ( sort keys %$hash ) {
-        $hash->{$k}->{display} = encode( 'utf8', $hash->{$k}->{name} );
-    }
-
     my $popup = Gtk2::Dialog->new( gettext('Scanning Results'),
         undef, 'destroy-with-parent' );
     $popup->signal_connect( destroy => sub { $popup->destroy } );
@@ -90,14 +96,13 @@ sub display {
         hover_expand    => TRUE
         );
 
-    # Display the possible infections
-    #for my $keys ( sort keys %$hash ) {
+    # Display the possible infections:
+    # for my $keys ( sort keys %$hash ) {
     my $i = 0;
     while ( $i <= scalar keys %$hash ) {
-        push @{ $slist->{data} }, [
-            #$hash->{$keys}->{display}, $hash->{$keys}->{status},
-            #$hash->{$keys}->{action},
-            $hash->{$i}->{display}, $hash->{$i}->{status},
+        push @{ $slist->{data} },
+            [
+            $hash->{$i}->{name}, $hash->{$i}->{status},
             $hash->{$i}->{action},
             ];
         $i++;
@@ -105,10 +110,6 @@ sub display {
         last if ( $i == scalar keys %$hash );
     }
 
-    my $tooltips = Gtk2::Tooltips->new;
-    $tooltips->set_tip( $slist,
-        gettext('Select a file and right-click for options...') );
-
     # Left-click will clear the top label
     $slist->signal_connect(
         button_press_event => sub {
@@ -126,13 +127,6 @@ sub display {
             my $deref = $sel[0];
             defined $deref or return;
 
-            # We have to avoid parsing mail for now:
-            # http://clamtk.sourceforge.net/faq.html#inbox
-            my $maildirs = join( '|',
-                '.thunderbird', '.mozilla-thunderbird',
-                '.evolution',   'Mail',
-                'kmail',        "\.pst" );
-
             my $menu = Gtk2::Menu->new();
 
             # Right-click 'quarantine' option
@@ -226,6 +220,33 @@ sub display {
     $bottombar->set_style('both-horiz');
     $pbox->pack_start( $bottombar, FALSE, FALSE, 0 );
 
+    my $quar_img = Gtk2::Image->new_from_stock( 'gtk-refresh', 'menu' );
+    my $quar_btn = Gtk2::ToolButton->new( $quar_img, gettext('Quarantine') );
+    $quar_btn->set_is_important(TRUE);
+    $bottombar->insert( $quar_btn, -1 );
+    $quar_btn->signal_connect(
+        clicked => sub {
+            my @sel = $slist->get_selected_indices;
+            for my $t (@sel) {
+                next if ( $hash->{$t}->{name} =~ /$maildirs/ );
+                next if ( -d $hash->{$t}->{name} );
+                main_confirm( $t, 'q' );
+            }
+        } );
+
+    my $del_close_btn = Gtk2::ToolButton->new_from_stock('gtk-delete');
+    $del_close_btn->set_is_important(TRUE);
+    $bottombar->insert( $del_close_btn, -1 );
+    $del_close_btn->signal_connect(
+        clicked => sub {
+            my @sel = $slist->get_selected_indices;
+            for my $u (@sel) {
+                next if ( $hash->{$u}->{name} =~ /$maildirs/ );
+                next if ( -d $hash->{$u}->{name} );
+                main_confirm( $u, 'd' );
+            }
+        } );
+
     my $b_sep = Gtk2::SeparatorToolItem->new;
     $b_sep->set_draw(FALSE);
     $b_sep->set_expand(TRUE);
@@ -254,6 +275,7 @@ sub main_confirm {
 
     #my $full_name = $slist->{data}[$number][0];
     my $full_name = $hash->{$number}->{name};
+    my $md5sum    = $hash->{$number}->{md5sum};
     my $status    = $slist->{data}[$number][1];
     my $action    = $slist->{data}[$number][2];
 
@@ -321,7 +343,7 @@ sub main_confirm {
 sub main_slist_delete {
     my $number = shift;
     #my $full_name = $slist->{data}[$number][0];
-    my $full_name = $hash->{$number}->{display};
+    my $full_name = $hash->{$number}->{name};
     my $status    = $slist->{data}[$number][1];
     my $action    = $slist->{data}[$number][2];
     $slist->{data}[$number][0] =
