@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2011 Dave M
+# ClamTk, copyright (C) 2004-2012 Dave M
 #
 # This file is part of ClamTk (http://clamtk.sourceforge.net).
 #
@@ -30,9 +30,12 @@ use POSIX 'locale_h', 'strftime';
 # Highlights certain fields required by clamav.net's form:
 my $required_field = "<span foreground = 'red'>*</span>";
 
-# file chosen by the user for analysis:
+# File chosen by the user for analysis:
 my $chosen = '';
 
+# Url we post to
+my $url = 'http://cgi.clamav.net/';
+
 # Main window:
 my $sub_window;
 my ( $name_entry,  $email_entry,  $desc_entry );
@@ -300,9 +303,6 @@ sub submit {
         }
     }
 
-    # URL we're posting to
-    my $url = 'http://cgi.clamav.net/sendvirus.cgi';
-
     # Our $ua
     my $ua = LWP::UserAgent->new;
 
@@ -316,6 +316,9 @@ sub submit {
     my $description = $desc_entry->get_text || '';
     my $sampleis    = ( $new_btn->get_active ) ? 'virus' : 'falsepositive';
 
+    # URL we're posting to
+    $url .= ( $sampleis eq 'virus' ) ? 'sendmalware.cgi' : 'sendfp.cgi';
+
     # Apparently this is necessary
     from_to( $sendername,  'ISO-8859-1', 'UTF-8' );
     from_to( $email,       'ISO-8859-1', 'UTF-8' );
